<?php

/**
 * @file
 * Contains \Drupal\block_field\Plugin\Field\FieldFormatter\BlockFieldFormatter.
 */

namespace Drupal\block_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;

/**
 * Plugin implementation of the 'block_field' formatter.
 *
 * @FieldFormatter(
 *   id = "block_field",
 *   label = @Translation("Block field"),
 *   field_types = {
 *     "block_field"
 *   }
 * )
 */
class BlockFieldFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    /** @var \Drupal\Core\Block\BlockManagerInterface $block_manager */
    $block_manager = \Drupal::service('plugin.manager.block');

    $elements = [];
    foreach ($items as $delta => $item) {
      $plugin_id = $item->id;
      $settings = $item->settings;

      // @see \Drupal\block\BlockViewBuilder::buildPreRenderableBlock
      // @see template_preprocess_block()
      $block_instance = $block_manager->createInstance($plugin_id, $settings);

      $elements[$delta] = [
        '#theme' => 'block',
        '#attributes' => [],
        '#configuration' => $block_instance->getConfiguration(),
        '#plugin_id' => $block_instance->getPluginId(),
        '#base_plugin_id' => $block_instance->getBaseId(),
        '#derivative_plugin_id' => $block_instance->getDerivativeId(),
        '#id' => $item->id,
        'content' => $block_instance->build(),
      ];
    }

    return $elements;
  }

}
